(window.webpackJsonp=window.webpackJsonp||[]).push([[6],{K7Wh:function(t,a,e){"use strict";e.r(a);var n=e("p0pE"),r=e.n(n),o=e("d6i3"),s=e.n(o),c=(e("miYZ"),e("tsqr")),i=e("IHB9"),u=e("Qyje");var p={namespace:"sign",state:{status:void 0},effects:{login:s.a.mark(function t(a,e){var n,r,o,p,d,w;return s.a.wrap(function(t){for(;;)switch(t.prev=t.next){case 0:return n=a.payload,r=e.call,o=e.put,t.next=4,r(i.a,n);case 4:return p=t.sent,t.next=7,o({type:"changeLoginStatus",payload:p});case 7:200===p.code?(c.a.success("登录成功！"),d=Object(u.parse)(window.location.href.split("?")[1]),(w=d.next)&&(window.location.href=w.startsWith("http")?w:window.location.origin+w)):c.a.error(p.msg);case 8:case"end":return t.stop()}},t)}),getCaptcha:s.a.mark(function t(a,e){var n,r;return s.a.wrap(function(t){for(;;)switch(t.prev=t.next){case 0:return n=a.payload,r=e.call,t.next=4,r(i.b,n);case 4:case"end":return t.stop()}},t)})},reducers:{changeLoginStatus:function(t,a){var e=a.payload;return function(t){var a="string"==typeof t?[t]:t;localStorage.setItem("antd-pro-authority",JSON.stringify(a));try{window.reloadAuthorized&&window.reloadAuthorized()}catch(t){}}(e.currentAuthority),r()({},t,{status:e.status,type:e.type})}}};a.default=p}}]);