.antd-pro-components-header-dropdown-index-container>*{background-color:#fff;border-radius:4px;box-shadow:0 6px 16px -8px rgba(0,0,0,.08),0 9px 28px 0 rgba(0,0,0,.05),0 12px 48px 16px rgba(0,0,0,.03)}@media screen and (max-width:480px){.antd-pro-components-header-dropdown-index-container{width:100%!important}.antd-pro-components-header-dropdown-index-container>*{border-radius:0!important}}
.antd-pro-components-select-lang-index-menu .anticon{margin-right:8px}.antd-pro-components-select-lang-index-menu .ant-dropdown-menu-item{min-width:160px}.antd-pro-components-select-lang-index-dropDown{line-height:64px;vertical-align:top;cursor:pointer}.antd-pro-components-select-lang-index-dropDown>i{font-size:16px!important;transform:none!important}.antd-pro-components-select-lang-index-dropDown>i svg{position:relative;top:-1px}
.antd-pro-components-global-header-index-logo{display:inline-block;height:64px;padding:0 0 0 24px;font-size:20px;line-height:64px;vertical-align:top;cursor:pointer}.antd-pro-components-global-header-index-logo img{display:inline-block;vertical-align:middle}.antd-pro-components-global-header-index-menu .anticon{margin-right:8px}.antd-pro-components-global-header-index-menu .ant-dropdown-menu-item{min-width:160px}.antd-pro-components-global-header-index-trigger{height:64px;padding:22px 24px;font-size:20px;cursor:pointer;transition:all .3s,padding 0s}.antd-pro-components-global-header-index-trigger:hover{background:rgba(0,0,0,.025)}.antd-pro-components-global-header-index-right{float:right;height:100%;overflow:hidden}.antd-pro-components-global-header-index-right .antd-pro-components-global-header-index-action{display:inline-block;height:100%;padding:0 12px;cursor:pointer;transition:all .3s}.antd-pro-components-global-header-index-right .antd-pro-components-global-header-index-action>i{color:rgba(0,0,0,.65);vertical-align:middle}.antd-pro-components-global-header-index-right .antd-pro-components-global-header-index-action:hover{background:rgba(0,0,0,.025)}.antd-pro-components-global-header-index-right .antd-pro-components-global-header-index-action.opened{background:rgba(0,0,0,.025)}.antd-pro-components-global-header-index-right .antd-pro-components-global-header-index-search{padding:0 12px}.antd-pro-components-global-header-index-right .antd-pro-components-global-header-index-search:hover{background:transparent}.antd-pro-components-global-header-index-right .antd-pro-components-global-header-index-account .antd-pro-components-global-header-index-avatar{margin:20px 8px 20px 0;color:#1890ff;vertical-align:top;background:hsla(0,0%,100%,.85)}.antd-pro-components-global-header-index-right .antd-pro-components-global-header-index-account .antd-pro-components-global-header-index-avatarHead{color:#000}.antd-pro-components-global-header-index-dark{height:64px}.antd-pro-components-global-header-index-dark .antd-pro-components-global-header-index-action{color:hsla(0,0%,100%,.85)}.antd-pro-components-global-header-index-dark .antd-pro-components-global-header-index-action>i{color:hsla(0,0%,100%,.85)}.antd-pro-components-global-header-index-dark .antd-pro-components-global-header-index-action.opened,.antd-pro-components-global-header-index-dark .antd-pro-components-global-header-index-action:hover{background:#1890ff}.antd-pro-components-global-header-index-dark .antd-pro-components-global-header-index-action .ant-badge{color:hsla(0,0%,100%,.85)}@media only screen and (max-width:768px){.ant-divider-vertical{vertical-align:unset}.antd-pro-components-global-header-index-name{display:none}i.antd-pro-components-global-header-index-trigger{padding:22px 12px}.antd-pro-components-global-header-index-logo{position:relative;padding-right:12px;padding-left:12px}.antd-pro-components-global-header-index-right{position:absolute;top:0;right:12px;background:#fff}.antd-pro-components-global-header-index-right .antd-pro-components-global-header-index-account .antd-pro-components-global-header-index-avatar{margin-right:0}}
