(window.webpackJsonp=window.webpackJsonp||[]).push([[14],{"9D7j":function(e,n,t){"use strict";t.r(n);var r=t("p0pE"),a=t.n(r),u=t("d6i3"),c=t.n(u),o=(t("miYZ"),t("tsqr")),s=t("jHYr"),i=t.n(s),p=t("io9h");function f(){return Object(p.a)("/api/count")}var d={namespace:"countPage",state:{namespaceNum:0,resourceNum:0,functionNum:0,fileNum:0,transformNum:0},effects:{fetch:c.a.mark(function e(n,t){var r,a,u;return c.a.wrap(function(e){for(;;)switch(e.prev=e.next){case 0:return i()(n),r=t.call,a=t.put,e.next=4,r(f);case 4:if(void 0===(u=e.sent)||!u.success){e.next=10;break}return e.next=8,a({type:"update",payload:void 0!==u?u:{}});case 8:e.next=11;break;case 10:o.a.error(u.msg);case 11:case"end":return e.stop()}},e)})},reducers:{update:function(e,n){return a()({},e,n.payload)}}};n.default=d},jHYr:function(e,n){e.exports=function(e){if(null==e)throw new TypeError("Cannot destructure undefined")}}}]);