(window.webpackJsonp=window.webpackJsonp||[]).push([[7],{Xfv2:function(e,t,n){e.exports={normal:"antd-pro-pages-resources-index.css-normal",baseInfo:"antd-pro-pages-resources-index.css-baseInfo"}},mB2O:function(e,t,n){"use strict";n.r(t);n("IzEo");var r=n("bx4M"),a=(n("jCWc"),n("kPKH")),c=(n("14J3"),n("BMrR")),o=(n("h7lp"),n("bf48")),u=n("2Taf"),l=n.n(u),s=n("vZ4D"),i=n.n(s),f=n("l4Ni"),p=n.n(f),m=n("ujKo"),d=n.n(m),b=n("MhPg"),h=n.n(b),v=n("q1tI"),y=n.n(v),E=n("Xfv2"),g=n.n(E),R=n("MuoO");function w(e){return function(){var t,n=d()(e);if(function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(e){return!1}}()){var r=d()(this).constructor;t=Reflect.construct(n,arguments,r)}else t=n.apply(this,arguments);return p()(this,t)}}var P=function(e,t,n,r){var a,c=arguments.length,o=c<3?t:null===r?r=Object.getOwnPropertyDescriptor(t,n):r;if("object"==typeof Reflect&&"function"==typeof Reflect.decorate)o=Reflect.decorate(e,t,n,r);else for(var u=e.length-1;u>=0;u--)(a=e[u])&&(o=(c<3?a(o):c>3?a(t,n,o):a(t,n))||o);return c>3&&o&&Object.defineProperty(t,n,o),o},j=function(e){h()(n,e);var t=w(n);function n(){return l()(this,n),t.apply(this,arguments)}return i()(n,[{key:"componentDidMount",value:function(){(0,this.props.dispatch)({type:"countPage/fetch"})}},{key:"render",value:function(){var e=this.props.count;return y.a.createElement(y.a.Fragment,null,y.a.createElement(o.a,{onBack:function(){return null},title:"资源统计",subTitle:"数据大屏"}),y.a.createElement("div",{className:g.a.baseInfo},y.a.createElement(c.a,{gutter:16},y.a.createElement(a.a,{span:6},y.a.createElement(r.a,{title:"命名空间",bordered:!1},e.namespaceNum)),y.a.createElement(a.a,{span:6},y.a.createElement(r.a,{title:"数据源",bordered:!1},e.resourceNum)),y.a.createElement(a.a,{span:6},y.a.createElement(r.a,{title:"函数",bordered:!1},e.functionNum)),y.a.createElement(a.a,{span:6},y.a.createElement(r.a,{title:"SQL",bordered:!1},e.transformNum)))))}}]),n}(v.Component),k=j=P([Object(R.connect)(function(e){return{count:e.countPage}})],j);t.default=k}}]);